@@ -99,6 +99,8 @@ struct _GnomeCanvasHacktextCharInfo {
 	ArtDRect bbox;
 };
 
+static GnomeCanvasHacktextCharInfo *gnome_canvas_hacktext_get_info (GnomeCanvasHacktext *hacktext, gint32 ch);
+
 struct _GnomeCanvasHacktextPriv {
 	Gt1LoadedFont *font;
 	double affine[6]; /* the item to world transform */
@@ -184,6 +186,9 @@ gnome_canvas_hacktext_init (GnomeCanvasHacktext *hacktext)
 	hacktext->text = NULL;
 	hacktext->priv = g_new (GnomeCanvasHacktextPriv, 1);
 	hacktext->priv->font = NULL;
+	hacktext->priv->affine[0] = hacktext->priv->affine[3] = 1.0;
+	hacktext->priv->affine[1] = hacktext->priv->affine[2] =
+	  hacktext->priv->affine[4] = hacktext->priv->affine[5] = 0.0;
 	for (i = 0; i < 256; i++)
 		hacktext->priv->info[i].bpath = NULL;
 }
@@ -211,6 +216,7 @@ gnome_canvas_hacktext_destroy (GtkObject *object)
 	for (i = 0; i < 256; i++)
 		if (hacktext->priv->info[i].bpath)
 			art_free (hacktext->priv->info[i].bpath);
+	g_free (hacktext->priv);
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -731,6 +737,7 @@ item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coord
 	{
 		char str[128];
 		art_affine_to_string (str, i2c);
+		g_print ("hacktext item_to_canvas %s\n", str);
 	}
 #endif
 
@@ -821,8 +828,10 @@ gnome_canvas_hacktext_point (GnomeCanvasItem *item, double mx, double my,
 	x = hacktext->x;
 	y = hacktext->y;
 	text = hacktext->text;
-	if (text == NULL)
+	if (text == NULL) {
+		/* This failed to return a value.  MW.  */
 		return 0.0;
+	}
 
 	art_affine_scale (size_affine, hacktext->size * 0.001, hacktext->size * -0.001);
 	art_affine_multiply (affine, size_affine, hacktext->priv->affine);
@@ -958,13 +967,8 @@ gnome_canvas_hacktext_req_repaint (GnomeCanvasHacktext *hacktext,
 
 	art_affine_scale (size_affine, hacktext->size * 0.001, hacktext->size * -0.001);
 	art_affine_multiply (affine, size_affine, hacktext->priv->affine);
-#if 0
 	affine[4] += x * hacktext->priv->affine[0] + y * hacktext->priv->affine[2];
 	affine[5] += x * hacktext->priv->affine[1] + y * hacktext->priv->affine[3];
-#else
-	affine [4] = x;
-	affine [5] = y;
-#endif
 	for (i = 0; text[i]; i++) {
 		info = gnome_canvas_hacktext_get_info (hacktext, text[i]);
 		art_drect_affine_transform (&w_bbox, &info->bbox, affine);
@@ -1016,29 +1020,26 @@ gnome_canvas_hacktext_render (GnomeCanvasItem *item,
 	if (text == NULL)
 		return;
 
+#ifdef VERBOSE
+	g_print ("buf->rect = (%d, %d) - (%d, %d)\n", buf->rect.x0, buf->rect.y0, buf->rect, buf->rect.y1);
+#endif
 	art_affine_scale (size_affine, hacktext->size * 0.001, hacktext->size * -0.001);
 	art_affine_multiply (affine, size_affine, hacktext->priv->affine);
-
-#if 0
 	affine[4] += x * hacktext->priv->affine[0] + y * hacktext->priv->affine[2];
 	affine[5] += x * hacktext->priv->affine[1] + y * hacktext->priv->affine[3];
-#endif
-	affine [4] = x;
-	affine [5] = y;
-#if 0
-	printf ("A %s -> (%g,%g) %g %g\n", text, x, y, affine [4], affine [5]);
-	printf ("f x' = x * %g + y * %g\ny' = x * %g y * %g\n",
-		hacktext->priv->affine [0],
-		hacktext->priv->affine [2],
-		hacktext->priv->affine [1],
-		hacktext->priv->affine [3]);
-#endif
 	for (i = 0; text[i]; i++) {
+#ifdef VERBOSE      
+		g_print ("character %c, affine = [%f %f %f %f %f %f]\n", text[i],
+			 affine[0], affine[1], affine[2], affine[3], affine[4], affine[5]);
+#endif
 		info = gnome_canvas_hacktext_get_info (hacktext, text[i]);
 		art_drect_affine_transform (&w_bbox, &info->bbox, affine);
 		art_drect_to_irect (&wi_bbox, &w_bbox);
+#ifdef VERBOSE
+		g_print ("wi_bbox = (%d, %d) - (%d, %d)\n", wi_bbox.x0, wi_bbox.y0, wi_bbox.x1, wi_bbox.y1);
+#endif
 		art_irect_intersect (&int_rect, &wi_bbox, &buf->rect);
-		if (!art_irect_empty (&int_rect)) { 
+		if (!art_irect_empty (&int_rect)) {
 			/* it intersects with the draw rect - draw the glyph */
 			affine_bpath = art_bpath_affine_transform (info->bpath, affine);
 			vpath = art_bez_path_to_vec (affine_bpath, 0.25);
