@@ -84,7 +84,7 @@ gnome_text_add_glyph_attr (int glyph_pos,
 
 
 GnomeTextLayout *
-gnome_text_layout_new (guchar *text, GnomeTextAttrEl *attrs)
+gnome_text_layout_new (const guchar *text, GnomeTextAttrEl *attrs)
 {
   GnomeTextLayout *layout;
   int byte_idx;
@@ -632,7 +632,7 @@ gnome_text_hs_just (GnomeTextLayout *layout)
       best_penalty = space_err * space_err + breaks[break_in_idx].penalty;
       best_idx = break_in_idx;
 
-      /* Now, keep trying to find a better break until either alll
+      /* Now, keep trying to find a better break until either all
 	 breaks are exhausted, or the maximum negative space
 	 constraint is violated, or the distance penalty is larger
 	 than the best total penalty so far. */
