@@ -21,7 +21,7 @@
 
 #define noVERBOSE
 
-int
+static int
 read_int32_lsb (const char *p)
 {
   const unsigned char *q = (unsigned char *)p;
@@ -128,7 +128,15 @@ struct _MyGt1String {
   char *fin;
 };
 
-Gt1TokenContext *
+static void
+tokenize_free (Gt1TokenContext *tc)
+{
+  gt1_free (tc->source);
+  gt1_free (tc);
+}
+
+
+static Gt1TokenContext *
 tokenize_new (const char *input)
 {
   Gt1TokenContext *tc;
@@ -144,7 +152,7 @@ tokenize_new (const char *input)
   return tc;
 }
 
-Gt1TokenContext *
+static Gt1TokenContext *
 tokenize_new_from_mystring (MyGt1String *input)
 {
   Gt1TokenContext *tc;
@@ -164,7 +172,7 @@ tokenize_new_from_mystring (MyGt1String *input)
 
    Note: this strips delimiters, like the initial /, and the enclosing ().
 */
-TokenType
+static TokenType
 tokenize_get (Gt1TokenContext *tc, MyGt1String *result)
 {
   char *s = tc->source;
@@ -327,7 +335,7 @@ ascii_to_hex (char c)
 /* return a hex byte, or -1 on error */
 
 /* we don't deal with comments here */
-int
+static int
 tokenize_get_hex_byte (Gt1TokenContext *tc)
 {
   char *s = tc->source;
@@ -363,7 +371,7 @@ tokenize_get_hex_byte (Gt1TokenContext *tc)
    This almost certainly implies changing the Gt1TokenContext structure
    to incorporate a size field, and de-emphasizing the use of
    zero-termination */
-void
+static void
 tokenize_get_raw (Gt1TokenContext *tc, char *buf, int buf_size)
 {
   memcpy (buf, tc->source + tc->index, buf_size);
@@ -448,7 +456,7 @@ struct _Gt1Proc {
 /* low-level PostScript routines */
 
 /* note: resulting array is _uninitialized_ ! */
-Gt1Array *
+static Gt1Array *
 array_new (Gt1Region *r, int size)
 {
   Gt1Array *array;
@@ -496,7 +504,8 @@ ensure_stack (Gt1PSContext *psc, int n)
     }
 }
 
-double parse_num (MyGt1String *number)
+static double
+parse_num (MyGt1String *number)
 {
   double sign;
   double mantissa;
@@ -801,7 +810,7 @@ eval_proc (Gt1PSContext *psc, Gt1Proc *proc)
 #endif
 }
 
-Gt1Value *
+static Gt1Value *
 gt1_dict_stack_lookup (Gt1PSContext *psc, Gt1NameId key)
 {
   int i;
@@ -817,7 +826,7 @@ gt1_dict_stack_lookup (Gt1PSContext *psc, Gt1NameId key)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_number (Gt1PSContext *psc, double *result, int index)
 {
   if (psc->n_values < index)
@@ -837,7 +846,7 @@ get_stack_number (Gt1PSContext *psc, double *result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_dict (Gt1PSContext *psc, Gt1Dict **result, int index)
 {
   if (psc->n_values < index)
@@ -857,7 +866,7 @@ get_stack_dict (Gt1PSContext *psc, Gt1Dict **result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_name (Gt1PSContext *psc, Gt1NameId *result, int index)
 {
   if (psc->n_values < index)
@@ -877,7 +886,7 @@ get_stack_name (Gt1PSContext *psc, Gt1NameId *result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_file (Gt1PSContext *psc, Gt1TokenContext **result, int index)
 {
   if (psc->n_values < index)
@@ -897,7 +906,7 @@ get_stack_file (Gt1PSContext *psc, Gt1TokenContext **result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_string (Gt1PSContext *psc, Gt1String *result, int index)
 {
   if (psc->n_values < index)
@@ -917,7 +926,7 @@ get_stack_string (Gt1PSContext *psc, Gt1String *result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_array (Gt1PSContext *psc, Gt1Array **result, int index)
 {
   if (psc->n_values < index)
@@ -937,7 +946,7 @@ get_stack_array (Gt1PSContext *psc, Gt1Array **result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_bool (Gt1PSContext *psc, int *result, int index)
 {
   if (psc->n_values < index)
@@ -957,7 +966,7 @@ get_stack_bool (Gt1PSContext *psc, int *result, int index)
 }
 
 /* return 1 on success, with result set to the top of the value stack */
-int
+static int
 get_stack_proc (Gt1PSContext *psc, Gt1Proc **result, int index)
 {
   if (psc->n_values < index)
@@ -1278,6 +1287,7 @@ internal_eexec (Gt1PSContext *psc)
       string.start = plaintext;
       string.fin = plaintext + plaintext_size;
       new_tc = tokenize_new_from_mystring (&string);
+      gt1_free (plaintext);
 
       if (psc->n_files_max == psc->n_files)
 	{
@@ -1520,7 +1530,7 @@ internal_closefile (Gt1PSContext *psc)
       else if (psc->file_stack[psc->n_files - 1] == tc)
 	{
 	  /* pop the file stack */
-	  /* todo: deallocate some of the tc context */
+	  tokenize_free (psc->tc);
 	  psc->n_files--;
 	  psc->tc = psc->file_stack[psc->n_files - 1];
 	  psc->n_values--;
@@ -1862,7 +1872,7 @@ InternalGt1ProcListing internal_procs[] = {
 
 /* here end the internal procedures */
 
-Gt1PSContext *
+static Gt1PSContext *
 pscontext_new (Gt1TokenContext *tc)
 {
   Gt1PSContext *psc;
@@ -1914,6 +1924,24 @@ pscontext_new (Gt1TokenContext *tc)
   return psc;
 }
 
+static void
+pscontext_free (Gt1PSContext *psc)
+{
+  /* Empty the value stack.  */
+  while (psc->n_values > 0) internal_pop (psc);
+  gt1_free (psc->value_stack);
+
+  gt1_free  (psc->file_stack);
+  gt1_free  (psc->gt1_dict_stack);
+
+  gt1_name_context_free (psc->nc);
+  gt1_region_free (psc->r);
+
+  gt1_free (psc);
+}
+
+
+
 static void
 eval_executable (Gt1PSContext *psc, Gt1Value *val)
 {
@@ -2085,7 +2113,7 @@ eval_ps (Gt1TokenContext *tc)
 
 /* This routine _assumes_ that plaintext is passed in with enough
    space to hold the decrypted text */
-void
+static void
 charstring_decrypt (Gt1String *plaintext, Gt1String *ciphertext)
 {
   int ciphertext_size;
@@ -2114,7 +2142,8 @@ charstring_decrypt (Gt1String *plaintext, Gt1String *ciphertext)
   plaintext->size = ciphertext->size - 4;
 }
 
-void
+#ifdef VERBOSE
+static void
 print_glyph_code (Gt1String *glyph_code)
 {
   Gt1String plaintext;
@@ -2212,10 +2241,11 @@ print_glyph_code (Gt1String *glyph_code)
   printf ("\n");
   gt1_free (plaintext.start);
 }
+#endif
 
 /* Gt1Dict is the toplevel font dict. This allocates a new string for the
    plaintext and stores it in body. */
-void
+static void
 get_subr_body (Gt1PSContext *psc, Gt1String *body, Gt1Dict *fontdict, int subr)
 {
   Gt1Value *private_val;
@@ -2467,7 +2497,7 @@ bs_end (BezState *bs)
   return bezpath;
 }
 
-ArtBpath *
+static ArtBpath *
 convert_glyph_code_to_begt1_path (Gt1PSContext *psc, Gt1String *glyph_code,
 				Gt1Dict *fontdict,
 				double *p_wx)
@@ -2488,7 +2518,7 @@ convert_glyph_code_to_begt1_path (Gt1PSContext *psc, Gt1String *glyph_code,
 
   /* stuff for flex */
   double flexbuf[6];
-  int flexptr;
+  int flexptr = -1;
 
   exe_stack_ptr = 0;
   exe_stack[0].start = gt1_alloc (glyph_code->size);
@@ -2794,7 +2824,7 @@ struct _KernPair {
 
 #define kern_pair_hash(n1,n2) ((n1) * 367 + (n2))
 
-MunchedFontInfo *
+static MunchedFontInfo *
 munch_font_info (Gt1PSContext *psc, Font_Info *fi)
 {
   MunchedFontInfo *mfi;
@@ -2831,6 +2861,14 @@ munch_font_info (Gt1PSContext *psc, Font_Info *fi)
   return mfi;
 }
 
+static void
+free_munched_font_info (MunchedFontInfo *mfi)
+{
+  gt1_free (mfi->kern_pair_table);
+  gt1_free (mfi);
+}
+
+
 /* allocate a new filename, same as the old one, but with the extension */
 static char *
 replace_extension (const char *filename, const char *ext)
@@ -2854,7 +2892,7 @@ replace_extension (const char *filename, const char *ext)
 }
 
 Gt1LoadedFont *
-gt1_load_font (char *filename)
+gt1_load_font (const char *filename)
 {
   Gt1LoadedFont *loaded_font;
   Gt1Dict *fontdict;
@@ -2887,6 +2925,7 @@ gt1_load_font (char *filename)
       pfb_size += bytes_read;
       gt1_double (pfb, char, pfb_size_max);
     }
+  fclose (f);
 
   /*
   fwrite (pfb, 1, pfb_size, stdout);
@@ -2905,6 +2944,12 @@ gt1_load_font (char *filename)
 	  flat[pfb_size] = 0;
 	}
     }
+  else
+    {
+      flat = gt1_new (char, 1);
+      *flat = 0;
+    }
+  gt1_free (pfb);
 
   /*
   printf ("%s", flat);
@@ -2916,14 +2961,19 @@ gt1_load_font (char *filename)
 
 
   tc = tokenize_new (flat);
+  gt1_free (flat);
   psc = eval_ps (tc);
+  tokenize_free (tc);
+
+  if (psc->fonts->n_entries != 1)
+    {
+      pscontext_free (psc);
+      return NULL;
+    }
 
   loaded_font = gt1_new (Gt1LoadedFont, 1);
   loaded_font->psc = psc;
   
-  if (psc->fonts->n_entries != 1)
-    return NULL;
-
   fontdict = psc->fonts->entries[0].val.val.dict_val;
   loaded_font->fontdict = fontdict;
 
@@ -3101,6 +3151,16 @@ gt1_load_font (char *filename)
   return loaded_font;
 }
 
+void
+gt1_unload_font (Gt1LoadedFont *font)
+{
+  pscontext_free (font->psc);
+  if (font->mfi) free_munched_font_info (font->mfi);
+
+  gt1_free (font->encoding);
+  gt1_free (font);
+}
+
 
 #if 1
 ArtBpath *
